import * as THREE from 'three';
import {
    REVISION
} from 'three';
import {
    OrbitControls
} from 'three/examples/jsm/controls/OrbitControls'
import Stats from 'stats.js'
import {
    Pane
} from 'tweakpane'


let debug = true;

export default class Sketch {
    constructor() {
        if (debug) console.log("Three Version: ", REVISION);
        // 初始化
        this.initParam();
        this.init();
        // 调试
        // this.initAxesHelp();
        this.initGridHelp();
        this.initController();
        this.render();
        this.loop();
        // Main
        this.generateCircular();
    }



    /// 对象
    generateCircular() {
        let box = new THREE.BoxGeometry(1, 1, 1);
        let material = new THREE.MeshBasicMaterial({
            color: 'blue',
        });
        let mesh = new THREE.Mesh(box, material);
        this.box = mesh;
        this.scene.add(this.box);

        var mat4 = new THREE.Matrix4()
        // 默认值单位矩阵
        // 1, 0, 0, 0,
        // 0, 1, 0, 0,
        // 0, 0, 1, 0,
        // 0, 0, 0, 1
        console.log('查看矩阵对象默认值', mat4.elements);

        /// set()方法
        /// 重置矩阵的元素值，执行.set()方法，本质上改变的就是矩阵elements属性值
        mat4.set(
            1, 0, 0, 5,
            0, 1, 0, 3,
            0, 0, 1, 9,
            0, 0, 0, 1
        )

        mat4.transpose();
        console.log('查看mat4的转置矩阵', mat4.elements);

    }

    loop() {
        const Delta = this.clock.getDelta();
        this.index = window.requestAnimationFrame(this.loop.bind(this));
        this.render();
    }
    initParam() {
        this.width = window.innerWidth;
        this.height = window.innerHeight;
        this.aspect = this.width / this.height;
        this.clock = new THREE.Clock();
    }
    init() {
        const {
            innerWidth,
            innerHeight
        } = window;
        this.aspect = innerWidth / innerHeight;
        // 场景
        this.scene = new THREE.Scene();
        this.scene.background = new THREE.Color('#eeeeee');
        // 摄像机
        this.camera = new THREE.PerspectiveCamera(75, this.aspect, 0.1, 1000);
        this.camera.position.set(0, 0, 10);
        this.camera.lookAt(0, 0, 0);
        // 渲染
        this.aspect = innerWidth / innerHeight;
        this.renderer = new THREE.WebGLRenderer({
            antialias: true,
            alpha: true,
        });
        this.renderer.setSize(innerWidth, innerHeight);
        this.renderer.render(this.scene, this.camera);
        this.renderer.setPixelRatio = window.Pixi
        document.body.appendChild(this.renderer.domElement);
    }
    initAxesHelp() {
        this.axesHelper = new THREE.AxesHelper(100);
        this.scene.add(this.axesHelper);
    }
    initGridHelp() {
        this.gridHelper = new THREE.GridHelper(300, 300);
        this.gridHelper.position.y = -0.5;
        this.gridHelper.position.x = -0.5;
        this.gridHelper.position.z = -0.5;
        this.scene.add(this.gridHelper);
    }
    initController() {
        const that = this;
        this.controller = new OrbitControls(this.camera, this.renderer.domElement);
        this.controller.addEventListener('change', function () {
            that.renderer.render(that.scene, that.camera);
        })
    }
    render() {
        this.renderer.render(this.scene, this.camera);
    }
}